var prgressCount = 0;
var interruptCount = 0;
var finishCount = 0;
var lastDangerFileId = -1;

function showCompleteNotify(fileId){
	chrome.downloads.search({id: fileId}, function (results){
		if(results.length === 0) return;
		var theFile = results[0];
		var filename = theFile.filename.slice(theFile.filename.lastIndexOf("\\") + 1);
		
		chrome.downloads.getFileIcon(fileId, {size: 32}, function(iconUrl) {
			var canvas = document.createElement("canvas");
			canvas.width = 80;
			canvas.height = 80;
			var ctx = canvas.getContext("2d");

			//draw a circle on the image
			ctx.beginPath();
			ctx.arc(40, 40, 30, 0, 2 * Math.PI);
			ctx.lineWidth = 5;
			ctx.strokeStyle = '#C3C3C3';
			ctx.stroke();

			var img = new Image();
			img.onload = function() { //when image is loaded
				ctx.drawImage(img, 24, 24); //center(24 = 80/2 - 32/2)
				var dataURL = canvas.toDataURL();
				
				chrome.notifications.create(fileId.toString(), {
					type: "basic", iconUrl: dataURL,
					title: chrome.i18n.getMessage("download_complete"),
					message: filename,
					buttons: [{title: chrome.i18n.getMessage("open_file")}, {title: chrome.i18n.getMessage("open_folder")}]
				});	
			};
			img.src = iconUrl;	//load image from url
		});
	});
}

function showInterruptNotify(fileId){
	chrome.downloads.search({id: fileId}, function (results){
		if(results.length === 0) return;
		var theFile = results[0];
		var filename = theFile.filename.slice(theFile.filename.lastIndexOf("\\") + 1);
		
		chrome.notifications.create(fileId.toString(), {
			type: "basic", iconUrl: "../img/alert80.png",
			title: chrome.i18n.getMessage("download_interrupt"),
			message: filename,
			buttons: [{title: chrome.i18n.getMessage("retry")},	{title: chrome.i18n.getMessage("remove")}]
		});	
	});
}

function showDangerNotify(fileId){
	chrome.downloads.search({id: fileId}, function (results){
		if(results.length === 0) return;
		var theFile = results[0];
		var filename = theFile.filename.slice(theFile.filename.lastIndexOf("\\") + 1);
		
		chrome.notifications.create(fileId.toString(), {
			type: "basic", iconUrl: "../img/danger80.png",
			title: chrome.i18n.getMessage("dangerous_file"),
			message: filename,
			buttons: [{title: chrome.i18n.getMessage("accept")}, {title: chrome.i18n.getMessage("remove")}]
		});	
	});
}

loadSetting(function(){
	
	if(options.hideDownloadShelf){
		chrome.downloads.setShelfEnabled(false);
	}

	chrome.downloads.onCreated.addListener(function (downloadInfo){
		if(options.showDownloadNotify){
			if(downloadInfo.state == "in_progress"){
				chrome.notifications.create(downloadInfo.id.toString(), {
					type: "basic", iconUrl: "../img/downloads80.png",
					title: chrome.i18n.getMessage("download_start"), message: "Unknown"
				});
			}

		}
	});

	
	//----------------handle notifications events----------------

	chrome.notifications.onButtonClicked.addListener(function (notificationId, btnIdx){
		//we use fileId as notificationId
		chrome.downloads.search({id: parseInt(notificationId)}, function (results){
			if(results.length === 0) return;
			var theFile = results[0];
			
			if(theFile.danger != "safe" && theFile.danger != "accepted"){ //Danger Notify
				if(btnIdx === 0){
					lastDangerFileId = theFile.id;
					chrome.tabs.create({url: "danger.html", selected: true});
				}else{
					chrome.downloads.erase({id: theFile.id});
				}				
			}else if(theFile.state == "complete"){ //Complete Notify
				if(btnIdx === 0){
					chrome.downloads.open(theFile.id);
				}else{
					chrome.downloads.show(theFile.id);
				}
			}else if(theFile.state == "interrupted"){ //Interrupted Notify
				if(btnIdx === 0){
					//retry		
					chrome.downloads.download({url: theFile.url});
					chrome.downloads.erase({id: theFile.id});
				}else{
					chrome.downloads.erase({id: theFile.id});
				}
			}
			clearNotify();
			chrome.notifications.clear(notificationId);	
		});
	});
	
	//----------------handle notifications events----------------
	

	chrome.downloads.onChanged.addListener(function(downloadInfo){
		
		//console.log(downloadInfo);		

		if(options.showDownloadNotify){
			//clear download notify when user cancel
			if(downloadInfo.error != null && downloadInfo.error.current == "USER_CANCELED"){
				chrome.notifications.clear(downloadInfo.id.toString());
			}

			
			if(downloadInfo.danger != null && downloadInfo.danger.current != "safe" && downloadInfo.danger.current != "accepted"){
				/*
				lastDangerFileId = downloadInfo.id;
				chrome.tabs.create({url: "danger.html", selected: true});
				*/
				showDangerNotify(downloadInfo.id);
			}
		}

		if(downloadInfo.filename != null && downloadInfo.filename.current != null){
			var theFileName = downloadInfo.filename.current;
			chrome.notifications.update(downloadInfo.id.toString(), {
				message: theFileName.slice(theFileName.lastIndexOf("\\") + 1)
			});
		}

		
		if(downloadInfo.state != null){
			if(downloadInfo.state.previous != null){ //handle delete file update
				if(downloadInfo.state.previous == "in_progress" && downloadInfo.state.current == "complete"){
					finishCount++;
					if(options.showDownloadNotify){
						showCompleteNotify(downloadInfo.id);
					}
				}else if(downloadInfo.state.current == "interrupted" && 
						//true || //just for test
						(downloadInfo.error == null || downloadInfo.error.current != "USER_CANCELED")){
					interruptCount++;
					if(options.showDownloadNotify){
						showInterruptNotify(downloadInfo.id);
					}
				}			
			}
		}
			
		if(options.showNotify){		
			updateNotify();	
		}
	}); 
	
});


function updateNotify(){
	chrome.downloads.search({}, function (results){
		prgressCount = 0;
		var hasDanger = false;
		
		for(var i = 0; i < results.length; i++){
			if(results[i].state != "interrupted" && results[i].state != "complete"){
				prgressCount++;
				//has danger file
				if(results[i].danger != null &&	
					results[i].danger != "safe" && 
					results[i].danger != "accepted" &&
					results[i].danger.current != "safe" &&
					results[i].danger.current != "accepted"){
					hasDanger = true;
				}
			}
		}
		/*
		console.log(prgressCount);
		console.log(finishCount);
		console.log(interruptCount);
		*/
		if(prgressCount === 0 && finishCount === 0 && interruptCount === 0){
			chrome.browserAction.setBadgeText({text: ""});
		}else{
			if(hasDanger || interruptCount > 0){
				chrome.browserAction.setBadgeBackgroundColor({color:"#FF0000"});
			}else if(finishCount > 0){
				chrome.browserAction.setBadgeBackgroundColor({color:"#EF6C00"});
			}else{
				chrome.browserAction.setBadgeBackgroundColor({color:"#009688"});
			}
			
			chrome.browserAction.setBadgeText({text: finishCount.toString() + "/" + (finishCount + prgressCount + interruptCount)});
		}			
	});
}


function clearNotify(){
	finishCount = 0;
	interruptCount = 0;
	updateNotify();
}

updateNotify();