allPanels.bookmarks = new Panel(function(){
	var theTree = this.tree;
	
	this.middleAction = function(){
		toPage("chrome://bookmarks/");
	}
	
	function openAll(tarId, background){
		chrome.bookmarks.getChildren(tarId, function (nodes){
			if(nodes.length <= 15 || confirm("Open " + nodes.length + " tabs ?")) {
				for(var i = 0; i < nodes.length; i++){
					if(typeof nodes[i].url !== "undefined"){
						if(background){
							chrome.tabs.create({url: nodes[i].url, selected: false}); //open tab in background
						}else{
							toPage(nodes[i].url);
						}
					}	
				}				
			}
		});
		
	}
	
	this.updateTree = function(keyword){
		keyword = keyword.toLowerCase().trim();
		var treeData = [];
		
		function addTo(tree, node){
			var subTree = {};
			subTree.text = node.title.escapeHtml();
			
			if(typeof(node.children) != "undefined"){ //is a folder
				subTree.children = [];
				subTree.data = {
					id: node.id,
					type: "folder"
				};
				for(var i = 0; i < node.children.length; i++){
					addTo(subTree.children, node.children[i]);
				}
				if(keyword != ""){
					if(subTree.children.length == 0){
						return; //remove empty folders in search result
					}else{
						subTree.state = {"opened" : true}; //expand all folders in search result
					}
				}
			}else{ //is a leaf node
				subTree.data = {id: node.id, url: node.url, type: "link"};
				subTree.icon = "chrome://favicon/size/16@1x/" + node.url;
				if(keyword != "" && node.title.toLowerCase().indexOf(keyword) == -1
								 && node.url.toLowerCase().indexOf(keyword) == -1){
					return;
				}
			}
			
			subTree.data.oriText = node.title;
			
			if(subTree.data.oriText.byteLen() > 25){ //use byteLen, because 中文 is larger
				subTree.data.showTooltip = true;
			}

			tree.push(subTree);				
		}
		
		chrome.bookmarks.getTree(function(rootNode){
			var treeNodes = rootNode[0].children;
			
			for(var i = 0; i < treeNodes.length; i++){
				addTo(treeData, treeNodes[i]);
			}
			
			for(var i = 0; i < treeData.length; i++){
				treeData[i].data.fixed = true;
				treeData[i].state = {"opened" : true};						
			}
			
			//add bookmarks link
			treeData.push({
				text: chrome.i18n.getMessage("show_bookmark_manager"),
				data : {url: "chrome://bookmarks/", type: "link", fixed: true},
				icon : "chrome://favicon/size/16@1x/chrome://bookmarks/"
			});
			
			theTree.jstree({
				'core' : {
					'data' : treeData,
					"check_callback" : function (operation, node, node_parent, node_position, more) {
						return node.data.fixed !== true && typeof node_parent.data !== "undefined" && node_parent.data.type === "folder";
					},
				},
				"conditionalselect" : function (node, event) {
					if(event.which == 1){
						if(node.data.type === 'link'){
							toPage(node.data.url);
						}
						this.toggle_node(node);						
					}
					return false;
				},	
				"contextmenu": {
					"items": function (node) {
						if(node.data.fixed === true) return{};
						if(node.data.type === "folder"){
							return {
								"open": {
									"label": chrome.i18n.getMessage("open_all"),
									"action": function (obj) {
										openAll(node.data.id, false);														
									}
								},								
								"rename": {
									"label": chrome.i18n.getMessage("rename"),
									"action": function (obj) {
										var newName = prompt(chrome.i18n.getMessage("rename_msg"), node.data.oriText);
										if(newName != null){
											chrome.bookmarks.update(node.data.id, {title: newName});
											theTree.jstree(true).rename_node(node, newName.escapeHtml());
											node.data.oriText = newName;
										}
									}
								},
								"delete": {
									"label": chrome.i18n.getMessage("delete"),
									"action": function (obj) {
										chrome.bookmarks.removeTree(node.data.id, function(){
											theTree.jstree(true).delete_node(node);
										});															
									}
								},
							};
						}else{ //bookmarks link
							return {
								"open": {
									"label": chrome.i18n.getMessage("open"),
									"action": function (obj) {
										chrome.tabs.create({url: node.data.url, selected: true});
									}
								},
								"background":{
									"label": chrome.i18n.getMessage("background"),
									"action": function (obj) {
										chrome.tabs.create({url: node.data.url, selected: false});
									}									
								},
								"rename": {
									"label": chrome.i18n.getMessage("rename"),
									"action": function (obj) {
										var newName = prompt(chrome.i18n.getMessage("rename_msg"), node.data.oriText);
										if(newName != null){
											chrome.bookmarks.update(node.data.id, {title: newName});
											theTree.jstree(true).rename_node(node, newName.escapeHtml());
											node.data.oriText = newName;
										}
									}
								},
								"editUrl": {
									"label": chrome.i18n.getMessage("edit_url"),
									"action": function (obj) {
										var newUrl = prompt(chrome.i18n.getMessage("edit_url_msg"), node.data.url);
										if(newUrl != null){
											chrome.bookmarks.update(node.data.id, {url: newUrl});
											node.data.url = newUrl;
										}
									}
								},							
								"delete": {
									"label": chrome.i18n.getMessage("delete"),
									"action": function (obj) {
										chrome.bookmarks.remove(node.data.id, function(){
											theTree.jstree(true).delete_node(node);
										});										
									}
								}
							};							
						}
					}
				},				
				"plugins": ["conditionalselect", "dnd", "contextmenu"] //P.S. the "search" plugin is too slow
			});
		});
	}
	
	theTree.on("click auxclick", '.jstree-anchor', function(e) {
		if(e.which == 2){ //mouse middle button
			e.preventDefault();
			var node = theTree.jstree(true).get_node(e.target);
			if(node.data.type === 'link'){
				chrome.tabs.create({url: node.data.url, selected: false}); //open tab in background
			}else{
				openAll(node.data.id, true);
			}					
		}
	}).on('move_node.jstree', function (e, data) {
		var parentId = theTree.jstree(true).get_node(data.parent).data.id;
		var position = data.position;
		
		//handle order move shift
		if(data.parent === data.old_parent && position > data.old_position){
			position++;
		}
		
		chrome.bookmarks.move(data.node.data.id, {parentId: parentId, index: position});
	}).on('hover_node.jstree', addTooltip);
	
	
});