allPanels.downloads = new Panel(function(){
	var theLimit = 150;
	var selfObj = this;
	var theTree = this.tree;
	var onRebuild = false;
	var checkList = {};
	
	var finishCount = 0;
	var interruptCount = 0;
	
	var backgroundPage = chrome.extension.getBackgroundPage();
	
	function rebuildTree() {
		if(onRebuild) {
			clearTimeout(onRebuild); 
		}				
		onRebuild = setTimeout(function(){	
			selfObj.tree.jstree("destroy");
			selfObj.updateTree(selfObj.oldKeyword);
		}, 100);	
	}
	
	function startUpdate(){
		setInterval(function(){
			for(var theFileId in checkList){
				if(checkList[theFileId]){
					(function(fileId){
						chrome.downloads.search({
							id: parseInt(fileId)
						}, function (files) {
							
							if (files.length) {
								var theFile = files[0];      
								if(theFile.state !== "in_progress" || theFile.paused){
									delete checkList[fileId];
									return;
								}
								var toPrint = getSizeText(theFile.bytesReceived);
								
								if(theFile.totalBytes > 0){
									$("#progress" + fileId).css("width", (theFile.bytesReceived / theFile.totalBytes * 100) + "%");

									toPrint += "/" + getSizeText(theFile.totalBytes);
									
									if(theFile.estimatedEndTime != null){
										toPrint += "(" + getElapsedText(new Date(theFile.estimatedEndTime).getTime() - new Date().getTime()) + ")";
									}
								}else{
									toPrint += "/?";
								}
								$("#text" + fileId).text(toPrint);	
							}
						})				
					})(theFileId);
				}
			}
		}, 500);
	}
	
	this.middleAction = function(){
		toPage("chrome://downloads/");
	}
	
	this.updateTree = function(keyword){
		finishCount += backgroundPage.finishCount;
		interruptCount += backgroundPage.interruptCount;	
		
		backgroundPage.clearNotify();
		
		var treeData = [];
		
		var inProgressTree = {
			state: {"opened" : true},
			text : chrome.i18n.getMessage("in_progress"),
			children: [],
			data: {state: "in_progress_tree"}
		};
		var pausedTree = {
			state: {"opened" : true},
			text : chrome.i18n.getMessage("paused"),
			children: [],
			data: {state: "paused_tree"}			
		};
		var interruptedTree = {
			state: {"opened" : true},
			text : chrome.i18n.getMessage("interrupted"),
			children: [],
			data: {state: "interrupted_tree"}
		};

		var completeTree = {
			state: {"opened" : true},
			text : chrome.i18n.getMessage("complete"),
			children: [],
			data: {state: "complete_tree"}
		};

		var notExistTree = {
			state: {"opened" : true},
			text : chrome.i18n.getMessage("complete_deleted"),
			children: [],
			data: {state: "not_exist_tree"}
		};
					
		function drawTree(){
			if(inProgressTree.children.length > 0) treeData.push(inProgressTree);
			if(pausedTree.children.length > 0) treeData.push(pausedTree);
			if(interruptedTree.children.length > 0) treeData.push(interruptedTree);
			if(completeTree.children.length > 0) treeData.push(completeTree);
			if(notExistTree.children.length > 0) treeData.push(notExistTree);
			
			if(interruptedTree.children.length > 0 || completeTree.children.length > 0 || notExistTree.children.length > 0){
				//add download manager link
				treeData.push({
					text: chrome.i18n.getMessage("clear_all"),
					data : {
						state: "clear"
					},
					icon : "../img/clear16.png"
				});					
			}
					
			//add download folder
			treeData.push({
				text: chrome.i18n.getMessage("open_download"),
				data : {
					state: "open_folder"
				},
				icon : "../img/downloadFolder16.png"
			});
			
			treeData.push({
				text: chrome.i18n.getMessage("show_download_manager"),
				data : {
					state: "link",
					url:"chrome://downloads/"
				},
				icon : "chrome://favicon/size/16@1x/chrome://downloads/"
			});	
			
			
			
			theTree.jstree({
				'core' : {
					'data' : treeData
				},
				"conditionalselect" : function (node, event) {
					if(event.which == 1){
						if(node.data != null){
							switch(node.data.state){
								case "in_progress":
									//do nothing
								break;
								
								case "in_danger":
									backgroundPage.lastDangerFileId = node.data.id;
									chrome.tabs.create({url: "danger.html", selected: true});
								break;

								
								case "paused":
									chrome.downloads.resume(node.data.id);
								break;	
								
								case "interrupted":
								case "complete_deleted":
									chrome.downloads.download({url: node.data.url});
									chrome.downloads.erase({id: node.data.id});
								break;		
									
								case "complete":
									chrome.downloads.open(node.data.id);
								break;
								
								case "clear":
									chrome.downloads.erase({state:"complete"}, function(){
										chrome.downloads.erase({state:"interrupted"}, rebuildTree);
									});
								break;
								
								case "link":
									toPage(node.data.url);
								break;	
								
								case "open_folder":
									chrome.downloads.showDefaultFolder();
								break;	
							}
							
						}
						this.toggle_node(node);						
					}
					return false;
				},
				"contextmenu": {
					"items": function (node) {
						var allActions = {
							"keep": {
								"label": chrome.i18n.getMessage("accept"),
								"action": function (obj) {
									backgroundPage.lastDangerFileId = node.data.id;
									chrome.tabs.create({url: "danger.html", selected: true});
								}
							},
							"pause": {
								"label": chrome.i18n.getMessage("pause"),
								"action": function (obj) {
									chrome.downloads.pause(node.data.id);
								}
							},
							"pause_all": {
								"label": chrome.i18n.getMessage("pause_all"),
								"action": function (obj) {
									for(var childId in node.children){
										var childNode = theTree.jstree(true).get_node(node.children[childId]);
										chrome.downloads.pause(childNode.data.id);
									}
								}
							},							
							"resume": {
								"label": chrome.i18n.getMessage("resume"),
								"action": function (obj) {
									chrome.downloads.resume(node.data.id);
								}
							},
							"resume_all": {
								"label": chrome.i18n.getMessage("resume_all"),
								"action": function (obj) {
									for(var childId in node.children){
										var childNode = theTree.jstree(true).get_node(node.children[childId]);
										chrome.downloads.resume(childNode.data.id);
									}
								}
							},
							"retry": {
								"label": chrome.i18n.getMessage("retry"),
								"action": function (obj) {
									chrome.downloads.download({url: node.data.url});
									chrome.downloads.erase({id: node.data.id});
								}
							},
							"retry_all": {
								"label": chrome.i18n.getMessage("retry_all"),
								"action": function (obj) {
									for(var childId in node.children){
										var childNode = theTree.jstree(true).get_node(node.children[childId]);
										chrome.downloads.download({url: childNode.data.url});
										chrome.downloads.erase({id: childNode.data.id});
									}
								}
							},
							"redownload": {
								"label": chrome.i18n.getMessage("redownload"),
								"action": function (obj) {
									chrome.downloads.download({url: node.data.url});
									chrome.downloads.erase({id: node.data.id});
								}
							},
							"redownload_all": {
								"label": chrome.i18n.getMessage("redownload_all"),
								"action": function (obj) {
									for(var childId in node.children){
										var childNode = theTree.jstree(true).get_node(node.children[childId]);
										chrome.downloads.download({url: childNode.data.url});
										chrome.downloads.erase({id: childNode.data.id});
									}
								}
							},
							"openFolder": {
								"label": chrome.i18n.getMessage("open_folder"),
								"action": function (obj) {
									chrome.downloads.show(node.data.id);
								}
							},
							"remove": {
								"label": chrome.i18n.getMessage("remove"),
								"action": function (obj) {
									chrome.downloads.erase({id: node.data.id});
								}
							},
							"remove_all": {
								"label": chrome.i18n.getMessage("remove_all"),
								"action": function (obj) {
									for(var childId in node.children){
										var childNode = theTree.jstree(true).get_node(node.children[childId]);
										chrome.downloads.erase({id: childNode.data.id});
									}						
								}
							},
							"delete_from_disk": {
								"label": chrome.i18n.getMessage("delete_from_disk"),
								"action": function (obj) {
									chrome.downloads.removeFile(node.data.id, function(){
										chrome.downloads.erase({id: node.data.id});
									});
								}
							},
							"delete_all_from_disk": {
								"label": chrome.i18n.getMessage("delete_all_from_disk"),
								"action": function (obj) {
									if(confirm(chrome.i18n.getMessage("delete_all_msg"))){
										node.children.forEach(function(childHtmlId){
											var childNode = theTree.jstree(true).get_node(childHtmlId);
											chrome.downloads.removeFile(childNode.data.id, function(){
												chrome.downloads.erase({id: childNode.data.id});
											});									
										});
									}
								}
							},
							"interrupt": {
								"label": chrome.i18n.getMessage("interrupt"),
								"action": function (obj) {
									chrome.downloads.cancel(node.data.id);
								}
							},
							"interrupt_all": {
								"label": chrome.i18n.getMessage("interrupt_all"),
								"action": function (obj) {
									for(var childId in node.children){
										var childNode = theTree.jstree(true).get_node(node.children[childId]);
										chrome.downloads.cancel(childNode.data.id);
									}	
								}
							},
							"interrupt_remove": {
								"label": chrome.i18n.getMessage("interrupt_remove"),
								"action": function (obj) {
									chrome.downloads.cancel(node.data.id, function(){
										chrome.downloads.erase({id: node.data.id});
									});
								}
							},
							"interrupt_remove_all": {
								"label": chrome.i18n.getMessage("interrupt_remove_all"),
								"action": function (obj) {							
									for(var childId in node.children){
										var childNode = theTree.jstree(true).get_node(node.children[childId]);
										chrome.downloads.cancel(childNode.data.id, function(){
											chrome.downloads.erase({id: childNode.data.id});
										});
									}	
								}
							},
							"copy_title": {
								"label": chrome.i18n.getMessage("copy_title"),
								"action": function (obj) {
									prompt(chrome.i18n.getMessage("copy_hint"), node.data.name);
								},
								"separator_before": true
							},
							"copy_url": {
								"label": chrome.i18n.getMessage("copy_url"),
								"action": function (obj) {
									prompt(chrome.i18n.getMessage("copy_hint"), node.data.url);
								}
							},
							"open_download": {
								"label": chrome.i18n.getMessage("open_download"),
								"action": function (obj) {
									chrome.downloads.showDefaultFolder();
								}
							}						
						}
						
						function createMenu(toShow, isAppend){
							var menu = {};
							for(var i = 0; i < toShow.length; i++){
								menu[toShow[i]] = allActions[toShow[i]];
							}
							if(isAppend){
								//add copy actions at end
								menu["copy_title"] = allActions["copy_title"];
								menu["copy_url"] = allActions["copy_url"];
							}
							return menu;
						}
						
						
						if(node.data != null){
							switch(node.data.state){
								case "in_progress":
									return createMenu(["pause", "interrupt", "interrupt_remove"], true);
								case "in_danger":
									return createMenu(["keep", "interrupt", "interrupt_remove"], true);
									
								case "in_progress_tree":
									return createMenu(["pause_all", "interrupt_all", "interrupt_remove_all"], false);
								case "paused":
									return createMenu(["resume","interrupt", "remove"], true);
								case "paused_tree":
									return createMenu(["resume_all", "interrupt_all", "remove_all"], false);
								case "interrupted":
									return createMenu(["retry", "remove"], true);
								case "interrupted_tree":
									return createMenu(["retry_all", "remove_all"], false);
								case "complete_deleted":									
									return createMenu(["redownload", "remove"], true);
									
								case "not_exist_tree":									
									return createMenu(["redownload_all", "remove_all"], false);
									
								case "complete":
									return createMenu(["openFolder", "remove", "delete_from_disk"], true);
								case "complete_tree":
									return createMenu(["open_download", "remove_all", "delete_all_from_disk"], false);

							}
						}
					}
				},
				
				"plugins": ["conditionalselect", "contextmenu"]
			});
			startUpdate();
		}
		
		chrome.downloads.search({
			query: [keyword],
			orderBy: ["-startTime"],
			limit: theLimit
			
		}, function(results) {
			
			var finishDisplayCount = 0;
			var interruptDisplayCount = 0;
			
			var iconLoadCount = 0;
			
			for(var i = 0; i < results.length; i++){
				var fileNode = {};
				var theFile = results[i];
				var fileId = theFile.id;
				var filename = theFile.filename.slice(theFile.filename.lastIndexOf("\\") + 1);
				
				fileNode.text = filename.escapeHtml();

				fileNode.data = {
					id: fileId,
					url: theFile.url,
					name: filename,
					path: theFile.filename,
					state: theFile.state,
					oriText: filename,
					showTooltip: filename.byteLen() > 25
				};
				
				if(fileNode.text != ""){			
					(function(tar, index) { //load icon for each files	
						chrome.downloads.getFileIcon(index, {size: 16},
						function(iconUrl) {
							tar.icon = iconUrl;
							iconLoadCount ++;
							if(iconLoadCount == results.length){ //when load finish
								drawTree();
							}
						});
					})(fileNode, theFile.id);
				}else{
					fileNode.text = "Unknown";
					fileNode.data.text = "Unknown";
					fileNode.icon = "../img/unknown16.png";
					iconLoadCount ++;
				}
				
				var drawBar = function(addInfo){
					var percent = 0, totalSizeText = "?";
					if (theFile.totalBytes > 0) { //known totalBytes
						percent = theFile.bytesReceived / theFile.totalBytes * 100;
						totalSizeText = getSizeText(theFile.totalBytes);
					}
					fileNode.text += '<br><div class="progressBox"><div class="progress" id="progress' + 
						fileId + '" style="width:' + percent + '%;"></div><label id="'+ "text" + fileId +'">' +
						getSizeText(theFile.bytesReceived) + "/" + totalSizeText + 
						addInfo + '</label></div>';
				}
				
				var drawDangerBar = function(){
					fileNode.text += '<br><div class="progressBox"><div class="progressDanger" id="progress' + 
						fileId + '" style="width:100%;"></div><label id="'+ "text" + fileId +'">' + 
						chrome.i18n.getMessage("dangerous_file") + '</label></div>';
				}
				
				switch(theFile.state){
					case "in_progress":
						if(theFile.paused){
							fileNode.data.state = "paused";	
							drawBar('(' + chrome.i18n.getMessage("state_paused") + ')');
							
							if(checkList[fileId]){
								delete checkList[fileId];								
							}
							pausedTree.children.push(fileNode);	
						} else if(theFile.danger != null && 
							theFile.danger != "safe" && 
							theFile.danger != "accepted" && 
							theFile.danger.current != "safe" && 
							theFile.danger.current != "accepted") {
								
							fileNode.data.state = "in_danger";	
							drawDangerBar();

							inProgressTree.children.push(fileNode);								
						}else{
							drawBar("");
							inProgressTree.children.push(fileNode);					
							checkList[fileId] = true;						
						}
					break;
						
					case "interrupted":
						if(interruptDisplayCount < interruptCount){
							fileNode.text = '<label class = "highlight">' + fileNode.text + '</label>';
							interruptDisplayCount++;
						}
					
						interruptedTree.children.push(fileNode);
					break;		
						
					case "complete":										
						if(theFile.exists){
							if(finishDisplayCount < finishCount){
								fileNode.text = '<label class = "highlight">' + fileNode.text + '</label>';
								finishDisplayCount++;
							}
							completeTree.children.push(fileNode);
						} else{
							fileNode.data.state = "complete_deleted";
							notExistTree.children.push(fileNode);
						}					

					break;
				}				
			}

			if(results.length == 0 || iconLoadCount == results.length) drawTree(); //no download results, just drawTree
		});			
	}
	
	//just rebuild tree (quick and dirty)
	chrome.downloads.onChanged.addListener(rebuildTree); 
	
	chrome.downloads.onErased.addListener(rebuildTree);
	
	theTree.on("click auxclick", '.jstree-anchor', function(e) {
		if(e.which == 2){ //mouse middle button
			e.preventDefault();
			var node = theTree.jstree(true).get_node(e.target);
			if(node.data != null ){
				switch(node.data.state){
					case "in_progress":
					case "in_danger":
						chrome.downloads.cancel(node.data.id);
					break;
					
					case "paused":
					case "interrupted":
					case "complete":
					case "complete_deleted":
						chrome.downloads.erase({id: node.data.id});
					break;		

					case "link":
						chrome.tabs.create({url: node.data.url, selected: false}); //open tab in background
					break;
				}
			}	
		}
	}).on('hover_node.jstree', addTooltip);
});