allPanels.extensions = new Panel(function(){
	var theTree = this.tree;
	var selfObj = this;
	var onRebuild = false;
	
	this.middleAction = function(){
		toPage("chrome://extensions/");
	}
	
	function rebuildTree() {
		if(onRebuild) {
			clearTimeout(onRebuild); 
		}				
		onRebuild = setTimeout(function(){
			selfObj.tree.jstree("destroy");
			selfObj.updateTree(selfObj.oldKeyword);		
		}, 100);	
	}
	
	this.updateTree = function(keyword){
		keyword = keyword.toLowerCase().trim();
		var treeData = [];
		
		var devTree={
			state: {"opened" : true},
			text : chrome.i18n.getMessage("devs"),
			children: []			
		}
		
		var appTree = {
			state: {"opened" : true},
			text : chrome.i18n.getMessage("apps"),
			children: []
		};	
		
		var enabledTree = {
			state: {"opened" : true},
			text : chrome.i18n.getMessage("enabled"),
			children: []
		};
		var disabledTree = {
			state: {"opened" : true},
			text : chrome.i18n.getMessage("disabled"),
			children: []
		};
		
		function drawTree(){
			if(devTree.children.length > 0) treeData.push(devTree);
			if(appTree.children.length > 0) treeData.push(appTree);
			if(enabledTree.children.length > 0) treeData.push(enabledTree);
			if(disabledTree.children.length > 0) treeData.push(disabledTree);
			//add extension manager link
			treeData.push({
				text: chrome.i18n.getMessage("show_extension_manager"),
				data : {
					url: "chrome://extensions/",
					type: "link"
				},
				icon : "chrome://favicon/size/16@1x/chrome://extensions/"
			});
			
			
			theTree.jstree({
				'core' : {
					'data' : treeData
				},
				"conditionalselect" : function (node, event) {
					if(event.which == 1){
						if(node.data != null){
							if(node.data.type === "link"){
								toPage(node.data.url);
							}else if(node.data.enabled){							
								if(node.data.type === "app"){
									chrome.management.launchApp(node.data.id);
								}else if(node.data.optionsUrl != ""){
									chrome.tabs.create({url: node.data.optionsUrl, selected: true});
								}
							}else{
								chrome.management.setEnabled(node.data.id, true, rebuildTree);								
							}
						} 
						this.toggle_node(node);						
					}
					return false;
				},
				"contextmenu": {
					"items": function (node) {
						var menu = {};
						if(node.data.type !== "link"){
							if(node.data.enabled){
								if(node.data.dev){				
									menu["reload"] = {
										"label": chrome.i18n.getMessage("reload"),
										"action": function (obj) {
											chrome.management.setEnabled(node.data.id, false, function(){
												chrome.management.setEnabled(node.data.id, true, rebuildTree);
											});
										}
									};	
								}	
								
								if(node.data.optionsUrl != ""){				
									menu["option"] = {
										"label": chrome.i18n.getMessage("option"),
										"action": function (obj) {
											chrome.tabs.create({url: node.data.optionsUrl, selected: true});
										}
									};	
								}
								
								menu["disable"] = {
									"label": chrome.i18n.getMessage("disable"),
									"action": function (obj) {
										node.data.enabled = false;
										chrome.management.setEnabled(node.data.id, false, rebuildTree);								
									}
								};	
									
							}else{
								menu["enable"] = {
									"label": chrome.i18n.getMessage("enable"),
									"action": function (obj) {
										node.data.enabled = true;
										chrome.management.setEnabled(node.data.id, true, rebuildTree);								
									}
								};									
							}
								
							menu["remove"] = {
								"label": chrome.i18n.getMessage("remove"),
								"action": function (obj) {
									chrome.management.uninstall(node.data.id, {showConfirmDialog: true});
								}								
							};
						}
						return menu;
					}
				},	
				
				"plugins": ["conditionalselect", "contextmenu"]
			});				
		}
		
		chrome.management.getAll(function (extensions){
			extensions.sort(
				function(a, b){
					return a.name.localeCompare(b.name);
				}
			)
			for(var i = 0; i < extensions.length; i++){	
				if(keyword != "" && extensions[i].name.toLowerCase().indexOf(keyword) == -1){
					continue; //do not add
				}
				if(extensions[i].type == "extension"){ //only add extensions
					var node = {};
					
					var icons = extensions[i].icons;
					
					if(typeof icons !== "undefined"){
						for(var j = 0; j < icons.length; j++){
							if (icons[j].size === 16) {
								node.icon = icons[j].url;
								break;
							}
						}					
					}
					
					if(typeof node.icon === "undefined"){ //can not find icon
						//use favicon instead
						node.icon = "chrome://favicon/size/16@1x/chrome-extension://" + extensions[i].id;
					}

					node.text = extensions[i].name.escapeHtml();
					
					node.data = {
						id: extensions[i].id,
						optionsUrl: extensions[i].optionsUrl,
						enabled: extensions[i].enabled,
						type: "extension",
						oriText: extensions[i].name,
						showTooltip: extensions[i].name.byteLen() > 25
					};

					if(extensions[i].installType == "development"){
						node.data.dev = true;
						if(!extensions[i].enabled){
							node.text += "(" + chrome.i18n.getMessage("disable") + ")";
						} 
						devTree.children.push(node);
					}else{
						if(extensions[i].enabled){
							enabledTree.children.push(node);
						}else{
							disabledTree.children.push(node);
						}						
					}

				}else if(typeof extensions[i].appLaunchUrl != "undefined"){ //is an app
					var node = {};
					node.icon = "chrome://favicon/size/16@1x/chrome-extension://" + extensions[i].id;
					node.text = extensions[i].name;
					
					node.data = {
						id: extensions[i].id,
						optionsUrl: extensions[i].optionsUrl,
						enabled: extensions[i].enabled,
						url: extensions[i].appLaunchUrl,
						type: "app"
					};

					if(extensions[i].installType == "development"){
						node.data.dev = true;
						if(!extensions[i].enabled){
							node.text += "(" + chrome.i18n.getMessage("disable") + ")";
						} 
						devTree.children.push(node);
					}else{
						if(extensions[i].enabled){
							appTree.children.push(node);
						}else{
							disabledTree.children.push(node);
						}
					}				
				}
			}
			drawTree();
		});
		
	};
	
	theTree.on("click auxclick", '.jstree-anchor', function(e) {
		if(e.which == 2){ //mouse middle button
			e.preventDefault();
			var node = theTree.jstree(true).get_node(e.target);
			if(node.data != null){
				if(node.data.type === "link"){
					chrome.tabs.create({url: node.data.url, selected: false});
				}else{
					chrome.management.uninstall(node.data.id, {showConfirmDialog: true});
				}
			}	
			return false;
		}
	}).on('hover_node.jstree', addTooltip);
/*	
	theTree.on('contextmenu', '.jstree-anchor', function (e) {
		e.preventDefault();  //disable contextmenu
		var node = theTree.jstree(true).get_node(e.target);
		if(node.data != null && node.data.type !== "link"){
			chrome.management.setEnabled(node.data.id, !node.data.enabled, rebuildTree);			
		}	
	});
*/
});
