allPanels.history = new Panel(function(){
	var theLimit = 150;
	var theTree = this.tree;
	
	this.middleAction = function(){
		toPage("chrome://history/");
	}
	
	this.updateTree = function(keyword){
		var treeData = [];
		
		chrome.history.search({
			text: keyword,
			maxResults: theLimit,
			startTime: 0
			
		}, function(results) {
			
			var curFolder;
			var folderTime = -1;
			for(var i = 0; i < results.length; i++){
				var hisNode = {};
				hisNode.text = results[i].title;
				if(hisNode.text == "") hisNode.text = results[i].url;
				
				var hisTime = new Date(results[i].lastVisitTime);
				hisTime.setHours(0);
				hisTime.setMinutes(0);
				hisTime.setSeconds(0);				
				hisTime.setMilliseconds(0);

				//if is initial or not same day
				if(folderTime == -1 || folderTime.toDateString() !== hisTime.toDateString()){
					//create new folder
					folderTime = hisTime;
					curFolder = {
						state: {"opened" : true},
						text: folderTime.toLocaleDateString(),
						children: [],
						data: {
							type: "folder", time: hisTime.getTime()
						}
					};
					
					var now = new Date();
					if(now.toDateString() == folderTime.toDateString()){
						curFolder.text += "(" + chrome.i18n.getMessage("today") +")";
					}else{
						now.setDate(now.getDate() - 1);
						if(now.toDateString() == folderTime.toDateString()){
							curFolder.text += "(" + chrome.i18n.getMessage("yesterday") +")";
						}
					}

					treeData.push(curFolder);	
				}
				
				hisNode.data = {
					type: "link",
					name: results[i].title,
					url: results[i].url,
					time: results[i].lastVisitTime,
					oriText: hisNode.text,
					showTooltip: hisNode.text.byteLen() > 25
				};
				
				hisNode.text = hisNode.text.escapeHtml();
				hisNode.icon = "chrome://favicon/size/16@1x/" + results[i].url;
				
				curFolder.children.push(hisNode);
			}

			
			//add history link
			treeData.push({
				text: chrome.i18n.getMessage("show_full_history"),
				data : {type: "link",url: "chrome://history/"},
				icon : "chrome://favicon/size/16@1x/chrome://history/"
			});
			
			//add clearhist link
			treeData.push({
				text: chrome.i18n.getMessage("clear_browser_data"),
				data : {type: "link", url: "chrome://settings/clearBrowserData"},
				icon : "../img/clear16.png"
			});			
			theTree.jstree({
				'core' : {
					'data': treeData,
					"check_callback": true
				},
				"search": {
					"show_only_matches" : true,
					"show_only_matches_children" : true
				},

				"conditionalselect" : function (node, event) {
					if(event.which == 1){
						if(node.data.type === "link"){
							toPage(node.data.url);
						}
						this.toggle_node(node);						
					}
					return false;
				},
				"contextmenu": {
					"items": function (node) {
						if(node.data.type === "folder"){
							return {
								"remove": {
									"label": chrome.i18n.getMessage("remove"),
									"action": function (obj) {
										chrome.history.deleteRange(
											{startTime: node.data.time, endTime: node.data.time + 86400000}, function(){
												theTree.jstree(true).delete_node(node);
											}
										);
									}
								}
							};
						}
						if(node.data.type === "link"){
							return {
								"copyTitle": {
									"label": chrome.i18n.getMessage("copy_title"),
									"action": function (obj) {
										prompt(chrome.i18n.getMessage("copy_hint"), node.data.name);
									}
								},
								"copyUrl": {
									"label": chrome.i18n.getMessage("copy_url"),
									"action": function (obj) {
										prompt(chrome.i18n.getMessage("copy_hint"), node.data.url);
									},
									"separator_after": true
								},								

								"remove": {
									"label": chrome.i18n.getMessage("remove"),
									"action": function (obj) {
										/*chrome.history.deleteUrl({url: node.data.url}, function(){
												theTree.jstree(true).delete_node(node);
											}
										);*/
										chrome.history.deleteRange(
											{startTime: node.data.time - 0.1, endTime: node.data.time + 0.1}, function(){
												theTree.jstree(true).delete_node(node);
											}
										);
									}
								}
							};
						}
					}
				},	
				"plugins": ["search", "conditionalselect", "contextmenu"]
			});
		});			
	}

	theTree.on("click auxclick", '.jstree-anchor', function(e) {
		if(e.which == 2){ //mouse middle button
			e.preventDefault();
			var node = theTree.jstree(true).get_node(e.target);
			if(node.data.type === "link"){
				chrome.tabs.create({url: node.data.url, selected: false}); //open tab in background
			}					
		}
	}).on('hover_node.jstree', function (e, data) {
		if(data.node.data.showTooltip){			
			$("#" + data.node.id).prop('title', data.node.data.oriText);					
		}
	});
/*
	theTree.on('contextmenu', '.jstree-anchor', function (e) {
		e.preventDefault();  //disable contextmenu
		//var node = theTree.jstree(true).get_node(e.target);
	});*/
});