var theSearchBar = $('<input></input>');
theSearchBar.attr("placeholder", chrome.i18n.getMessage("search_hint"));
theSearchBar.css("width", "100%");

String.prototype.byteLen = function() { 
	var arr = this.match(/[^\x00-\xff]/ig); 
	return arr == null ? this.length : this.length + arr.length; 
} 

String.prototype.escapeHtml = function() { 
    return this
         .replace(/&/g, "&amp;")
         .replace(/</g, "&lt;")
         .replace(/>/g, "&gt;")
         .replace(/"/g, "&quot;")
         .replace(/'/g, "&#039;");	
} 
 
function getElapsedText(ms){
	ms = parseInt(ms);
	if(ms <= 0) return "0" + chrome.i18n.getMessage("sec") + " ";
	
	var text = "";
	var sec, min, hour, day, totalTime;
    totalTime = ms / 1000;
    
	sec = Math.floor(totalTime % 60);
    totalTime /= 60;
    
	min = Math.floor(totalTime % 60);
    totalTime /= 60;
	
    hour = Math.floor(totalTime % 24);
    totalTime /= 24;
	
    day = Math.floor(totalTime);

	if(day != 0) {
		text += day + chrome.i18n.getMessage("day");
	} else if(hour != 0){
		text += hour + chrome.i18n.getMessage("hour") + " " + min + chrome.i18n.getMessage("min");
	}else{
		if(min != 0) text += min + chrome.i18n.getMessage("min") + " ";
		text += sec + chrome.i18n.getMessage("sec");
	}

	return text;
}
 
function getSizeText(bytes){
	if(bytes < 1000) return bytes + "B";
	
	bytes = Math.floor(bytes / 1000); //to KB
	if(bytes < 1000) return bytes + "KB";
	
	bytes = Math.floor(bytes / 100) / 10; //to MB
	if(bytes < 1000) return bytes.toFixed(1) + "MB";
	
	bytes = Math.floor(bytes / 100) / 10; //to GB
	if(bytes < 1000) return bytes.toFixed(1) + "GB";	
}

function toPage(page) {
	chrome.tabs.getAllInWindow(null, function(tabs) {
		for (var i = 0; i < tabs.length; i++) {
			console.log(tabs[i].url);
			if (tabs[i].url === page) {
				chrome.tabs.update(tabs[i].id, {selected: true}, function(){
					window.close();
				});
				return;
			}
		}
		chrome.tabs.create({url: page, selected: true}, function(){
			window.close();
		});
	});	
}

function addTooltip(e, data) {
	if(data.node.data && data.node.data.showTooltip){
		var	tarNode = $("#" + data.node.id);
		tarNode.prop('title', data.node.data.oriText);	
	}
}

//abstract object, must implement "updateTree(keyword)" in initial
function Panel(initial){
	this.container = $("<div></div>");
	this.container.hide();	
	this.visible = false;

	this.hasInital = false;
	this.initial = initial;
}

Panel.prototype.hide = function(){
	this.container.hide();
	this.visible = false;
}

Panel.prototype.show = function(){
	if(this.hasInital === false){
		var theTree = $("<div></div>");	
		this.tree = theTree;
		
		var theTreeContainer = $('<div class = "panelContainer">');
		theTreeContainer.append(theTree);
		
		this.container.append(theTreeContainer);
		
		theTree.on("click auxclick", '.jstree-anchor', function(e) {
			theSearchBar.focus();
		});
		
		var selfObj = this;
		this.onSearch = false;
		
		var curSearch = theSearchBar.val();
		this.oldKeyword = curSearch;
		
		theSearchBar.keyup(function(e) {
			if(selfObj.onSearch) {
				clearTimeout(selfObj.onSearch); 
			}
			selfObj.onSearch = setTimeout(function(){
				selfObj.doSearch();
			}, 250);				
		});
		
		this.initial();
		this.updateTree(curSearch); //show the tree
		
		this.hasInital = true;
	}
	
	this.container.show();
	theSearchBar.focus();
	this.visible = true;
}


Panel.prototype.appendTo = function(tar){
	this.container.appendTo(tar);
}

Panel.prototype.doSearch = function () {
	var searchString = theSearchBar.val();
	if(this.oldKeyword == searchString) return;
	//console.log(searchString);
	this.tree.jstree("destroy");
	this.updateTree(searchString); //call abstract method
	this.oldKeyword = searchString;
}
